import argparse
import json
import socket
import time
from typing import List

import requests
from vsw.utils import Constant
import vsw.utils
from vsw.log import Log
from multiprocessing.connection import Listener

logger = Log(__name__).logger
timeout = Constant.TIMEOUT


def main(args: List[str]) -> bool:
    args = parse_args(args)
    print(args)
    try:
        if args.connection:
            connection_repo()
        if args.credential_definition:
            do_credential_definition(args.schema)
    except requests.exceptions.RequestException:
        logger.error("Please check if you have executed 'vsw setup' to start agent!")
    except KeyboardInterrupt:
        print(" ==> Exit init")


def get_schema_id_by_name(vsw_config, schema_name):
    if schema_name == vsw_config.get("test_schema_name"):
        return vsw_config.get("test_schema_id")
    else:
        return vsw_config.get("schema_id")


def do_credential_definition(schema_name):
    vsw_config = vsw.utils.get_vsw_agent()
    schema_id = get_schema_id_by_name(vsw_config, schema_name)

    credential_definition_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/credential-definitions'
    res = requests.post(credential_definition_url, json={
        "revocation_registry_size": 100,
        "support_revocation": True,
        "schema_id": schema_id,
        "tag": "default"
    })
    credential_definition_res = json.loads(res.text)
    logger.info(f'Created credential definition id: {credential_definition_res["credential_definition_id"]}')


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-cd", "--credential-definition", action='store_true')
    parser.add_argument('-c', '--connection', action='store_true')
    parser.add_argument('-s', '--schema', default='software-certificate', help="The schema name")

    return parser.parse_args(args)


def connection_repo():
    address = ('localhost', Constant.PORT_NUMBER)
    listener = Listener(address)
    listener._listener._socket.settimeout(Constant.TIMEOUT)
    vsw_config = vsw.utils.get_vsw_agent()
    remove_history_connection(vsw_config)
    vsw_repo_config = vsw.utils.get_repo_host()
    vsw_repo_url = f'{vsw_repo_config.get("host")}/connections/create-invitation?alias={vsw_repo_config.get("label")}&auto_accept=true'
    logger.debug(f'Create invitation to: {vsw_repo_url}')
    response = requests.post(vsw_repo_url)
    res = json.loads(response.text)
    logger.debug(res)

    local_url = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections/receive-invitation?alias={vsw_config.get("label")}'
    logger.debug(f'Receive invitation {local_url}')
    invitation = res["invitation"]
    body = {
        "label": invitation["label"],
        "serviceEndpoint": invitation["serviceEndpoint"],
        "recipientKeys": invitation["recipientKeys"],
        "@id": invitation["@id"]
    }
    ss = requests.post(local_url, json=body)
    invitation_response = json.loads(ss.text)
    logger.debug(invitation_response)

    connection_id = invitation_response["connection_id"]
    while True:
        try:
            conn = listener.accept()
            msg = conn.recv()
            state = msg["state"]
            conn.close()
            logger.info(f'waiting state change, current state is: {state}')
            if state == 'active':
                logger.info("Created connection successfully!")
                listener.close()
                break
            else:
                time.sleep(0.5)
        except socket.timeout:
            remove_connection(connection_id, vsw_config)
            logger.error("Request timeout, there might be some issue during initializing connection.")
            listener.close()
            break;


def remove_history_connection(vsw_config):
    schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections'
    response = requests.get(schema_url)
    results = json.loads(response.text)["results"]
    for result in results:
        schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections/{result["connection_id"]}/remove'
        requests.post(schema_url)
        logger.debug(f"Removed history connection id: {result['connection_id']}")


def remove_connection(connection_id, vsw_config):
    schema_url = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/connections/{connection_id}/remove'
    requests.post(schema_url)



import argparse
import calendar
from datetime import datetime, timezone
import json
import socket
import time
from multiprocessing.connection import Listener
from typing import List
from urllib import parse
from urllib.parse import urljoin

import requests
import validators
from aries_cloudagent_vsw.messaging.util import str_to_epoch
from version_parser import Version
from vsw.utils import Constant
import vsw.utils
from vsw.log import Log
from vsw.commands import attest

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = Constant.TIMEOUT


def main(args: List[str]) -> bool:
    try:
        parser = argparse.ArgumentParser(prog="vsw verify")
        parser.add_argument("-p", "--proof-request", required=True, help="The software credential json file path")
        parser.add_argument("-d", "--revoke-date", required=False, help="The revoke date, %Y-%m-%d")
        parsed_args = parser.parse_args(args)
        execute(parsed_args.proof_request, parsed_args.revoke_date)
    except requests.exceptions.RequestException:
        logger.error("Please check if you have executed 'vsw setup' to start agent!")
    except KeyboardInterrupt:
        print(" ==> Exit verify!")


def execute(proof_request, revoke_date):
    with open(proof_request) as json_file:
        data = json.load(json_file)
        software_credential = get_software_credential(data["requested_attributes"])
        test_credential = get_test_credential(data["requested_attributes"])
        if software_credential:
            if "attr::softwareversion::value" in software_credential:
                software_version = software_credential["attr::softwareversion::value"]
                if check_version(software_version) is False:
                    return;
            if "attr::softwareurl::value" in software_credential:
                software_url = software_credential["attr::softwareurl::value"]
                if not validators.url(software_url):
                    logger.info('The software package url is wrong, please check')
                    return
            credentials = check_credential(software_credential)
            if len(credentials) == 0:
                logger.info("No found matched credential, please check if the specified conditions are correct.")
                return;
        if test_credential:
            credentials = check_credential(test_credential)
            if len(credentials) == 0:
                logger.info("No found matched attest credential, please check if the specified conditions are correct.")
                return;
        requested_predicates = {}
        if "requested_predicates" in data:
            requested_predicates = data["requested_predicates"]
        connection = get_client_connection()
        logger.info("Executing verify, please wait for response...")
        logger.debug(f'issuer connection_id: {connection["connection_id"]}')
        address = ('localhost', Constant.PORT_NUMBER)
        listener = Listener(address)
        listener._listener._socket.settimeout(Constant.TIMEOUT)
        proof_response = send_request(connection["connection_id"], software_credential, test_credential, requested_predicates, revoke_date)
        presentation_exchange_id = proof_response["presentation_exchange_id"]
        logger.info(f'presentation_exchange_id: {presentation_exchange_id}')

        while True:
            try:
                conn = listener.accept()
                msg = conn.recv()
                state = msg["state"]
                conn.close()
                logger.info(f"waiting state update, current state is: {state}")
                if state == "verified":
                    if msg["verified"] == "false":
                        remove_proof_request(presentation_exchange_id)
                        logger.info("Verified error, Verified result from indy is False!")
                        listener.close()
                        break;
                    pres_req = msg["presentation_request"]
                    is_proof_of_software_certificate = (
                            pres_req["name"] == "Proof of Software Certificate"
                    )
                    if is_proof_of_software_certificate:
                        logger.info('Congratulation! verified successfully!')
                    else:
                        remove_proof_request(presentation_exchange_id)
                        logger.info("Verified error, the name in presentation request is wrong")
                    listener.close()
                    break;
                else:
                    time.sleep(0.5)
            except socket.timeout:
                remove_proof_request(presentation_exchange_id)
                logger.error("Request timeout, Verified error!")
                listener.close()
                break;


def get_software_credential(data):
    for cred in data:
        if cred["schema_id"] == vsw_config.get("schema_id"):
            return cred
    return None


def get_test_credential(data):
    for cred in data:
        if cred["schema_id"] == vsw_config.get("test_schema_id"):
            return cred
    return None


def check_version(software_version):
    try:
        Version(software_version)
    except ValueError:
        logger.info("The software version format is incorrect. the correct format should be 'MAJOR.MINOR.PATCH'")
        return False
    return True


def check_credential(data):
    wql = json.dumps(data)
    repo_url = f"{repo_url_host}/credentials?wql={parse.quote(wql)}"
    res = requests.get(repo_url)
    return json.loads(res.text)["results"]


def remove_proof_request(presentation_exchange_id):
    vsw_url = urljoin(vsw_url_host, f"/present-proof/records/{presentation_exchange_id}/remove")
    requests.post(vsw_url)


def retrieve_result(presentation_exchange_id):
    time.sleep(1)  # wait communicate complete automatically between agents
    presentation_proof_result = get_vsw_proof(presentation_exchange_id)
    return presentation_proof_result


def get_vsw_proof(pres_ex_id):
    vsw_url = urljoin(vsw_url_host, f"/present-proof/records/{pres_ex_id}")
    res = requests.get(vsw_url)
    return json.loads(res.text)


def send_request(client_conn_id, software_credential, test_credential, requested_predicates, revoke_date):
    vsw_url = f'{vsw_url_host}/present-proof/send-request'
    NOW_8601 = datetime.utcnow().replace(tzinfo=timezone.utc).isoformat(" ", "seconds")
    NOW_EPOCH = str_to_epoch(NOW_8601)
    time_from = 0
    time_to = NOW_EPOCH

    if revoke_date:
        time_to = calendar.timegm(datetime.strptime(revoke_date, '%Y-%m-%d').timetuple())
    req_attr = {
        "names": ["softwarename", "softwareversion", "developerdid", "softwaredid", "softwarehash", "softwareurl",
                  "mediatype", "sourcedid", "sourceurl", "sourcehash", "buildertooldidlist", "dependencydidlist",
                  "buildlog", "builderdid"],
        "non_revoked": {"from": time_from, "to": time_to},
        "restrictions": [software_credential]
    }
    test_names = ["testerdid", "testspecdid", "testspecurl", "testspechash", "testresult","testresultdetaildid",
                  "testresultdetailurl", "testresultdetailhash", "comments", "softwaredid"]
    if not requested_predicates or len(requested_predicates.values()) == 0:
        test_names.append("ranking")
    req_test_attr = {
        "names": test_names,
        "non_revoked": {"from": time_from, "to": time_to},
        "restrictions": [test_credential]
    }

    request_attributes = {}
    if software_credential:
        request_attributes["0_software_certificate_uuid"] = req_attr
    if test_credential:
        request_attributes["1_test_certificate_uuid"] = req_test_attr
    if requested_predicates:
        for p in requested_predicates.values():
            p["restrictions"] = [{"cred_def_id": attest.get_credential_definition_id()}]
            p["non_revoked"] = {"from": time_from, "to": time_to}

    indy_proof_request = {
        "name": "Proof of Software Certificate",
        "version": "1.0",
        "requested_attributes": request_attributes,
        "requested_predicates": requested_predicates
    }
    proof_request_web_request = {
        "connection_id": client_conn_id,
        "proof_request": indy_proof_request
    }
    logger.debug(proof_request_web_request)
    res = requests.post(vsw_url, json=proof_request_web_request)
    return json.loads(res.text)


def get_client_connection():
    connection_response = requests.get(f'{vsw_url_host}/connections?state=active')
    res = json.loads(connection_response.text)
    connections = res["results"]
    if len(connections) > 0:
        return connections[-1]
    else:
        raise ConnectionError("Not found active vsw connection! Have you executed vsw init -c?")



import argparse
import json
from typing import List
from urllib.parse import urljoin
from rich.console import Console
import requests

from vsw.log import Log
from vsw.utils import get_vsw_agent, get_repo_host

logger = Log(__name__).logger
console = Console()


def main(argv: List[str]) -> bool:
    args = parse_args(argv)
    vsw_config = get_vsw_agent()
    vsw_repo_config = get_repo_host()
    repo_url_host = vsw_repo_config.get("host")
    try:
        if args.connection:
            get_connections(vsw_config)
        elif args.wallet:
            get_wallet(vsw_config)
        elif args.schema:
            get_schema(vsw_config)
        elif args.status:
            get_status(vsw_config)
        elif args.credentials:
            get_credentials(repo_url_host, vsw_config)
        elif args.credential_definition:
            get_credential_definition(vsw_config)
        elif args.present_proof:
            get_present_proof(vsw_config)
        else:
            console.print('Usage:')
            console.print('vsw list [options]')
            console.print('-c: list all connections')
            console.print('-w: list all DIDs in the wallet')
            console.print('-sc: list all supported schema')
            console.print('-s: show agent status')
            console.print('-p: list all presentation proof records')
            console.print('-cs: list all credentials issued by the current DID')
            console.print('-cd: list all credential definitions registered by the current DID')
    except KeyboardInterrupt:
        print(" ==> Exit list!")
    except requests.exceptions.RequestException:
        logger.error("Please check if you have executed 'vsw setup' to start agent!")
    except Exception as e:
        logger.error("Failed to execute list: " + str(e))


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--connection', action='store_true')
    parser.add_argument('-w', '--wallet', action='store_true')
    parser.add_argument('-sc', '--schema', action='store_true')
    parser.add_argument('-s', '--status', action='store_true')
    parser.add_argument('-p', '--present_proof', action='store_true')
    parser.add_argument('-cs', '--credentials', action='store_true')
    parser.add_argument('-cd', '--credential_definition', action='store_true')
    return parser.parse_args(args)


def get_credential_definition(vsw_config):
    did = get_public_did(vsw_config)
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/credential-definitions/created?issuer_did={did}'
    response = requests.get(local)
    res = json.loads(response.text)
    console.print(res)


def get_credentials(repo_url_host, vsw_config):
    did = get_public_did(vsw_config)
    repo = urljoin(f'{repo_url_host}', '/credentials?wql={"attr::developerdid::value":"'+did+'"}')
    response = requests.get(repo)
    res = json.loads(response.text)
    console.print(res)


def get_issue_credential_records(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}/issue-credential/records'
    response = requests.get(local)
    res = json.loads(response.text)
    console.print(res)


def get_public_did(vsw_config):
    url = urljoin(f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}', "/wallet/did/public")
    response = requests.get(url)
    res = json.loads(response.text)
    return res["result"]["did"]


def get_status(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/status'
    response = requests.get(local)
    res = json.loads(response.text)
    console.print(res)


def get_schema(vsw_config):
    schema_id = vsw_config.get("schema_id")
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/schemas/{schema_id}'
    response = requests.get(local)
    res = json.loads(response.text)
    console.print(f"======vsw software certificate schema_id: {schema_id}======")
    console.print(res)

    test_schema_id = vsw_config.get("test_schema_id")
    test_local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/schemas/{test_schema_id}'
    test_response = requests.get(test_local)
    test_res = json.loads(test_response.text)
    console.print(f"======vsw attest schema_id: {test_schema_id}======")
    console.print(test_res)


def get_wallet(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/wallet/did'
    response = requests.get(local)
    res = json.loads(response.text)
    console.print(res)


def get_present_proof(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/present-proof/records'
    response = requests.get(local)
    res = json.loads(response.text)
    console.print(res)


def get_connections(vsw_config):
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/connections'
    response = requests.get(local)
    res = json.loads(response.text)
    console.print(res)




import argparse
import json
import socket
import time
from typing import List
from vsw.log import Log
from urllib.parse import urljoin
import vsw.utils
from urllib import parse
import requests
import validators
from vsw.utils import Constant
from version_parser import Version
from multiprocessing.connection import Listener
from vsw.commands import attest

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = Constant.TIMEOUT


def main(args: List[str]) -> bool:
    try:
        args = parse_args(args)
        with open(args.cred_file) as json_file:
            data = json.load(json_file)
            logger.debug(f'schema name: {args.schema}')
            if args.schema == vsw_config.get("test_schema_name"):
                attest.publish(data)
            else:
                if "softwareName" not in data:
                    print("softwareName is mandatory")
                    return
                if "softwareVersion" not in data:
                    print("softwareVersion is mandatory")
                    return
                if "softwareUrl" not in data:
                    print("softwareUrl is mandatory")
                    return
                if "softwareVersion" in data:
                    software_version = data["softwareVersion"]
                    if check_version(software_version) is False:
                        return;
                if "softwareUrl" in data:
                    software_url = data["softwareUrl"]
                    if software_url and not validators.url(software_url):
                        print('The software package url is wrong, please check')
                        return
                issue_credential(data)
    except requests.exceptions.RequestException:
        logger.error("Please check if you have executed 'vsw setup' to start agent!")
    except ValueError as ve:
        logger.error(ve)
    except KeyboardInterrupt:
        print(" ==> Exit publish!")


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-c", "--cred-file", required=True, help="The software credential json file path")
    parser.add_argument('-s', '--schema', default='software-certificate', help="The schema name")
    return parser.parse_args(args)


def check_version(software_version):
    try:
        Version(software_version)
    except ValueError:
        print("The software version format is incorrect. the correct format should be 'MAJOR.MINOR.PATCH'")
        return False
    return True


def issue_credential(data):
    logger.info("executing publish, please waiting for response")
    address = ('localhost', Constant.PORT_NUMBER)
    listener = Listener(address)
    listener._listener._socket.settimeout(Constant.TIMEOUT)
    proposal_response = send_proposal(data)
    credential_exchange_id = proposal_response["credential_exchange_id"]
    logger.info(f'credential_exchange_id: {credential_exchange_id}')
    while True:
        try:
            conn = listener.accept()
            msg = conn.recv()
            state = msg["state"]
            logger.info(f'waiting state change, current state is: {state}')
            conn.close()
            if state == 'credential_acked':
                logger.info("Congratulation, execute publish successfully!")
                listener.close()
                break
            else:
                time.sleep(0.5)
        except socket.timeout:
            remove_credential(credential_exchange_id)
            logger.error("Request timeout, there might be some issue during publishing")
            listener.close();
            break;


def remove_credential(credential_exchange_id):
    url = urljoin(repo_url_host, f"/issue-credential/records/{credential_exchange_id}/remove")
    requests.post(url)


def get_repo_connection():
    vsw_connection_response = requests.get(f'{vsw_url_host}/connections?state=active')
    res = json.loads(vsw_connection_response.text)
    connections = res["results"]
    if len(connections) > 0:
        last_connection = connections[-1]
        repo_connection_response = requests.get(
            f'{repo_url_host}/connections?state=active&my_did={last_connection["their_did"]}&their_did={last_connection["my_did"]}')
        repo_res = json.loads(repo_connection_response.text)
        repo_connections = repo_res["results"]
        if len(repo_connections) > 0:
            return repo_connections[-1]
        else:
            raise ConnectionError("Not found related repo active connection!")
    else:
        raise ConnectionError("Not found active vsw connection! Have you executed vsw init -c?")


def get_public_did():
    url = urljoin(vsw_url_host, "/wallet/did/public")
    response = requests.get(url)
    res = json.loads(response.text)
    return res["result"]["did"]


def get_credential(developer_did, software_name):
    wql = json.dumps({"attr::developerdid::value": developer_did, "attr::softwarename::value": software_name})
    repo_url = f"{repo_url_host}/credentials?wql={parse.quote(wql)}"
    res = requests.get(repo_url)
    try:
        return json.loads(res.text)["results"]
    except BaseException:
        return None


def is_same_version(software_version, exist_software_version):
    if exist_software_version is None:
        return False
    v1 = Version(software_version)
    v2 = Version(exist_software_version)
    if v1.get_major_version() == v2.get_major_version() and v1.get_minor_version() == v2.get_minor_version():
        return True
    else:
        return False


def generate_software_did(developer_did, software_name, software_version, download_url):
    credentials = get_credential(developer_did, software_name)
    if len(credentials) > 0:
        logger.info(f'The software name {software_name} is existed.')
        same_version = False
        for credential in credentials:
            attrs = credential["attrs"]
            same_version = is_same_version(software_version, attrs["softwareVersion"])
            if same_version:
                logger.info(f'Existed softwareDid: {attrs["softwareDid"]}')
                return attrs["softwareDid"]
        if same_version is False:
            return generate_new_did(download_url)
    else:
        return generate_new_did(download_url)


def generate_new_did(download_url):
    # Create a DID
    create_did_res = requests.post(urljoin(vsw_url_host, "/wallet/did/create"))
    res = json.loads(create_did_res.text)
    did = res["result"]["did"]
    verkey = res["result"]["verkey"]
    # Write DID to ledger by NYM
    ledger_res = requests.post(urljoin(vsw_url_host, f"/ledger/register-nym?did={did}&verkey={verkey}"))
    write_did_ledger_res = json.loads(ledger_res.text)
    if write_did_ledger_res["success"] is False:
        logger.err("write did to ledger failed!")
        raise Exception('write did to ledger failed!')
    # Set DID Endpoint
    did_endpoint_res = requests.post(urljoin(vsw_url_host, f"/wallet/set-did-endpoint"), json={
        "did": did,
        "endpoint_type": "Endpoint",
        "endpoint": download_url  # not support :h1 format
    })
    logger.info(f'Created new software-did: {did}')
    return did


def get_credential_definition_id():
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/credential-definitions/created?schema_id={vsw_config.get("schema_id")}'
    response = requests.get(local)
    res = json.loads(response.text)
    if len(res["credential_definition_ids"]) == 0:
        raise ValueError('Not found credential definition id!')
    cred_def_id = res["credential_definition_ids"][-1]
    logger.debug(f'cred_def_id: {cred_def_id}')
    return cred_def_id


def send_proposal(data):
    developer_did = get_public_did()
    connection = get_repo_connection()
    logger.debug(f'holder connection_id: {connection["connection_id"]}')

    digest = vsw.utils.generate_digest(data["softwareUrl"])
    source_hash = vsw.utils.generate_digest(data["sourceUrl"])
    if "sourceHash" in data and data["sourceHash"] != source_hash:
        raise ValueError("Incorrect sourceHash value!")
    cred_def_id = get_credential_definition_id()
    software_did = generate_software_did(developer_did, data["softwareName"], data["softwareVersion"], data["softwareUrl"])
    vsw_repo_url = f'{repo_url_host}/issue-credential/send-proposal'
    res = requests.post(vsw_repo_url, json={
        "comment": "execute vsw publish cli",
        "auto_remove": False,
        "trace": True,
        "connection_id": connection["connection_id"],
        "schema_id": data["schemaID"] or vsw_config.get("schema_id"),
        "schema_name": data["schemaName"] or vsw_config.get("schema_name"),
        "schema_version": data["schemaVersion"] or vsw_config.get("schema_version"),
        "issuer_did": developer_did,
        "cred_def_id": cred_def_id,
        "credential_proposal": {
            "@type": f"did:sov:{developer_did};spec/issue-credential/1.0/credential-preview",
            "attributes": [
                {
                    "name": "developerDid",
                    "value": developer_did
                },
                {
                    "name": "softwareName",
                    "value": data["softwareName"]
                },
                {
                    "name": "softwareVersion",
                    "value": data["softwareVersion"]
                },
                {
                    "name": "softwareDid",
                    "value": software_did
                },
                {
                    "name": "softwareUrl",
                    "value": data["softwareUrl"]
                },
                {
                    "name": "softwareHash",
                    "value": digest
                },
                {
                    "name": "mediaType",
                    "value": data["mediaType"]
                },
                {
                    "name": "sourceDid",
                    "value": data["sourceDid"]
                },
                {
                    "name": "sourceUrl",
                    "value": data["sourceUrl"]
                },
                {
                    "name": "sourceHash",
                    "value": source_hash
                },
                {
                    "name": "builderToolDidList",
                    "value": data["builderToolDidList"]
                },
                {
                    "name": "dependencyDidList",
                    "value": data["dependencyDidList"]
                },
                {
                    "name": "buildLog",
                    "value": data["buildLog"]
                },
                {
                    "name": "builderDid",
                    "value": data["builderDid"]
                }
            ]
        },
    })
    return json.loads(res.text)



import json

from vsw.commands import verify


def test_main():
    verify.execute("/Users/Felix/development/vsw-workspace/vsw/dist/verify.json")


def test_retrieve_results():
    verify.retrieve_result("01168b93-ea56-452e-b7ca-e14d4be1bfca")


def test_check_credential():
    verify.check_credential("1234", "python")


def test_get_software_credential():
    with open("/Users/Felix/development/vsw-workspace/vsw/dist/verify.json") as json_file:
        data = json.load(json_file)
        credential = verify.get_test_credential(data)
        print(credential)
        soft_credential = verify.get_software_credential(data)
        print(soft_credential)


import argparse
import json
import socket
import time
from multiprocessing.connection import Listener
from typing import List
from urllib.parse import urljoin

import requests
import validators

import vsw.utils
from vsw.log import Log
from vsw.utils import Constant

vsw_config = vsw.utils.get_vsw_agent()
vsw_repo_config = vsw.utils.get_repo_host()
vsw_url_host = f'http://{vsw_config.get("admin_host")}:{vsw_config.get("admin_port")}'
repo_url_host = vsw_repo_config.get("host")
logger = Log(__name__).logger
timeout = Constant.TIMEOUT


def main(args: List[str]) -> bool:
    try:
        args = parse_args(args)
        with open(args.attest_file) as json_file:
            data = json.load(json_file)
            publish(data)
    except requests.exceptions.RequestException:
        logger.error("Please check if you have executed 'vsw setup' to start agent!")
    except KeyboardInterrupt:
        print(" ==> Exit attest!")


def publish(data):
    if "testSpecUrl" in data:
        test_spec_url = data["testSpecUrl"]
        if test_spec_url and not validators.url(test_spec_url):
            print('The testSpecUrl is wrong, please check')
            return
    if "testResultDetailUrl" in data:
        test_result_detail_url = data["testResultDetailUrl"]
        if test_result_detail_url and not validators.url(test_result_detail_url):
            print('The testResultDetailUrl is wrong, please check')
            return
    if "testSpecDid" in data or "testSpecUrl" in data:
        if "testResult" not in data:
            print("The testResult is mandatory if specify testSpecDid or testSpecUrl")
            return
    if "testSpecDid" not in data and "testSpecUrl" not in data and "ranking" not in data:
        print("The rank is mandatory if not specify testSpec")
        return
    issue_credential(data)


def parse_args(args):
    parser = argparse.ArgumentParser()
    parser.add_argument("-a", "--attest-file", required=True, help="The attestation credential json file path")
    return parser.parse_args(args)


def get_credential_definition_id():
    local = f'http://{vsw_config.get("admin_host")}:{str(vsw_config.get("admin_port"))}/credential-definitions/created?schema_id={vsw_config.get("test_schema_id")}'
    response = requests.get(local)
    res = json.loads(response.text)
    cred_def_id = res["credential_definition_ids"][-1]
    logger.info(f'cred_def_id: {cred_def_id}')
    return cred_def_id


def issue_credential(data):
    logger.info("executing publish, please waiting for response")
    address = ('localhost', Constant.PORT_NUMBER)
    listener = Listener(address)
    listener._listener._socket.settimeout(Constant.TIMEOUT)
    proposal_response = send_proposal(data)
    credential_exchange_id = proposal_response["credential_exchange_id"]
    logger.info(f'credential_exchange_id: {credential_exchange_id}')

    while True:
        try:
            conn = listener.accept()
            msg = conn.recv()
            state = msg["state"]
            logger.info(f'waiting state change, current state is: {state}')
            conn.close()
            if state == 'credential_acked':
                logger.info("Congratulation, execute publish successfully!")
                listener.close()
                break
            else:
                time.sleep(0.5)
        except socket.timeout:
            remove_credential(credential_exchange_id)
            logger.error("Request timeout, there might be some issue during publishing")
            listener.close()
            break;


def remove_credential(credential_exchange_id):
    url = urljoin(repo_url_host, f"/issue-credential/records/{credential_exchange_id}/remove")
    requests.post(url)


def get_repo_connection():
    vsw_connection_response = requests.get(f'{vsw_url_host}/connections?state=active')
    res = json.loads(vsw_connection_response.text)
    connections = res["results"]
    if len(connections) > 0:
        last_connection = connections[-1]
        repo_connection_response = requests.get(
            f'{repo_url_host}/connections?state=active&my_did={last_connection["their_did"]}&their_did={last_connection["my_did"]}')
        repo_res = json.loads(repo_connection_response.text)
        repo_connections = repo_res["results"]
        if len(repo_connections) > 0:
            return repo_connections[-1]
        else:
            raise ConnectionError("Not found related repo active connection!")
    else:
        raise ConnectionError("Not found active vsw connection! Have you executed vsw init -c?")


def get_public_did():
    url = urljoin(vsw_url_host, "/wallet/did/public")
    response = requests.get(url)
    res = json.loads(response.text)
    return res["result"]["did"]


def send_proposal(data):
    tester_did = get_public_did()
    connection = get_repo_connection()
    logger.info(f'holder connection_id: {connection["connection_id"]}')

    test_spec_hash = vsw.utils.generate_digest(data["testSpecUrl"])
    test_result_detail_hash = vsw.utils.generate_digest(data["testResultDetailUrl"])
    vsw_repo_url = f'{repo_url_host}/issue-credential/send-proposal'
    res = requests.post(vsw_repo_url, json={
        "comment": "execute vsw publish cli",
        "auto_remove": False,
        "trace": True,
        "connection_id": connection["connection_id"],
        "schema_id": data["schemaID"] or vsw_config.get("test_schema_id"),
        "schema_name": data["schemaName"] or vsw_config.get("test_schema_name"),
        "schema_version": data["schemaVersion"] or vsw_config.get("test_schema_version"),
        "issuer_did": tester_did,
        "cred_def_id": get_credential_definition_id(),
        "credential_proposal": {
            "@type": f"did:sov:{tester_did};spec/issue-credential/1.0/credential-preview",
            "attributes": [
                {
                    "name": "testerDid",
                    "value": tester_did
                },
                {
                    "name": "softwareDid",
                    "value": data["softwareDid"]
                },
                {
                    "name": "testSpecDid",
                    "value": data["testSpecDid"]
                },
                {
                    "name": "testSpecUrl",
                    "value": data["testSpecUrl"]
                },
                {
                    "name": "testSpecHash",
                    "value": test_spec_hash
                },
                {
                    "name": "testResult",
                    "value": data["testResult"]
                },
                {
                    "name": "testResultDetailDid",
                    "value": data["testResultDetailDid"]
                },
                {
                    "name": "testResultDetailUrl",
                    "value": data["testResultDetailUrl"]
                },
                {
                    "name": "testResultDetailHash",
                    "value": test_result_detail_hash
                },
                {
                    "name": "ranking",
                    "value": data["ranking"]
                },
                {
                    "name": "comments",
                    "value": data["comments"]
                }
            ]
        },
    })
    return json.loads(res.text)



import os
import subprocess
from typing import List

from vsw.log import Log

logger = Log(__name__).logger


def main(args: List[str]) -> bool:
    kill_lt()
    kill_controller()
    kill_vsw()
    print("Exited vsw")


def kill_vsw():
    out = os.popen("ps aux | grep setup").read()
    for line in out.splitlines():
        if 'setup' in line:
            pid = int(line.split()[1])
            subprocess.run(f'kill -9 {pid}', stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=True)


def kill_lt():
    out = os.popen("ps aux | grep npx").read()
    for line in out.splitlines():
        if 'localtunnel' in line:
            pid = int(line.split()[1])
            subprocess.run(f'kill -9 {pid}', stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=True)


def kill_controller():
    out = os.popen("ps aux | grep controller/server.py").read()
    for line in out.splitlines():
        if 'python3' in line:
            pid = int(line.split()[1])
            subprocess.run(f'kill -9 {pid}', stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, shell=True)